;
; Line timings difference between VGA and NTSC using IRQ_LINE to change backdrop color. This test is for real hardware only
; and doesn't produce any difference between the video modes or correct timings on the current emulators!
;
; Programmed by: doslogo
; Copyright (C) 2025 doslogo
;
; Disclaimer: The code below is written to get a test going as quick as possible to run on the real hardware, and the code is not 
;             stable to use in any game without heavy register management and synchronization methods! This code is not meant as 
;             a teaching case!
;

.define USE_GREEN_COLOR		1	; Uses green backdrop instead of red if that is preferred

SCNALINE_TEST			= (118*2); Scanline number to test (we want an even number because 240p, but this is a value between 0-511)

; Keep the X coordinate relatively close to the beginning of the scanline so that we can see the rendering of the current Y scanline
X_COORDINATE_LINE		= 12	; This is kinda outside the overscan though


NUM_NOPS				= 169	;63+little for VGA, 169 for NTSC

.segment "CODE"
	cld							; Clear decimal mode

	stz $9F25					; DCSEL=0 for this entire test!

	;
	; Let's set VGA or NTSC-240p based on the preprocessor definition VGA:
	;
.if VGA=0
	lda #%01001010				; 240p, NTSC output, and sprites
.else
	lda #%01001001				; 240p, VGA, and sprites
.endif

	sta $9F29

	; 240p means 320x240 resolution!
	lda #$40
	sta $9F2A
	sta $9F2B

	;
	; Get an ISR up to do vblank and hblank for us
	;
	php
	sei							; Disable interrupts

	lda #<isr
	sta $0314+0
	lda #>isr
	sta $0314+1

	; Turn on vblank, vblank will turn on hblank
	lda #%00000001
	sta $9F26

	;
	; Need a sprite on the scanline we are interested in, because it is difficult to count scanlines on a regular CRT with composite.
	; Turn on the mouse, and force its coordinates to the scanline we care about
	;
	lda #1
	ldx #0
	ldy #0
	jsr $FF68

	stz $9F25
	lda #$02					; Sprite 0 X coordinate address $1FC02
	sta $9F20					; 
	lda #$FC
	sta $9F21
	lda #(%00010000|$01)		; Increment 1 byte and high bit of VRAM address
	sta $9F22

	lda #<X_COORDINATE_LINE		; X coordinate of mouse cursor tip
	sta $9F23
	lda #>X_COORDINATE_LINE		; X coordinate of mouse cursor tip
	sta $9F23
	lda #0						; Y coordinate of mouse cursor tip (240p so only the low byte)
	sta $9F23

	plp							; Restore interrupts

	;
	; No more to do here, everything is done with the ISR!
	;


@:
	inc _are_we_doing_game_cycles
	bra @-

_are_we_doing_game_cycles:	.byte 0
_test_col:		.byte 0
_scanline_240p:	.byte 0
;
; Interrupt Service Routine:
;
isr:
	; Act fast, only recognize vblank, else it is hblank (this might not be true)!
	lda $9F27
	and #%00000001
	beq @hblank

	jmp @vblank

@hblank:
	.if ADD_NOPS=1
.repeat NUM_NOPS
	nop
.endrepeat
	.endif

	inc _test_col
	lda _test_col
	and #$01
	bne @set_col

	; Set the backdrop color to black. This is the timing visual trigger!
	stz $9F23
	stz $9F23

	bra @+

@set_col:
	stz $9F23
	lda #$0F
	sta $9F23

	; Time to stop lines yet?
	lda _test_col
	cmp #(22*2)					; Do 22 red lines
	bcc @+

	; Disable hblank, only do vblank
	lda #%00000001
	sta $9F26

@:
	stz $9F25					; ADDRSEL=0
	stz $9F20					; Reset backdrop color address $1FA00

	; Set next +2 scanline (and get the scanline for the new Y position of the sprite)
	lda $9F28
	sta _scanline_240p
	clc
	adc #2
	sta $9F28
	lda $9F26
	and #%01000000
	bne @+

	bcc @+						; Wow, from the adc above

	lda $9F26
	ora #%10000000				; Write different from read!
	sta $9F26

@:
	cmp #%01000000				; Get the carry
	lda _scanline_240p
	ror				 			; Convert to 240p (using the carry bit as the high bit)
	sta $9F24					; Y coordinate of mouse cursor tip (240p so only the low byte)

@exit_isr:
	; Ack IRQs!
	lda #%00000011				; Signal VERA we are done with this IRQ
	sta $9F27

	;
	; Return from interrupt (skipping kernal, so no keyboard, no turning off led etc!)
	;
	ply
	plx
	pla
	rti

@vblank:
	;
	; Hblank must have priority and speed for this test, so set VERA addresses now
	;
	stz $9F25
	stz $9F20					; backdrop color address $1FA00
	lda #$FA
	sta $9F21
	lda #(%00010000|$01)		; Increment 1 byte and high bit of VRAM address
	sta $9F22

	; Set the address for the sprite Y position
	lda #1						; ADDRSEL=1
	sta $9F25
	lda #$04					; Sprite 0 Y coordinate address $1FC02
	sta $9F20					; 
	lda #$FC
	sta $9F21
	lda #$01					; No increment and high bit of VRAM address
	sta $9F22

.if 0;;;
	; Set the backdrop color to something visible, like red
.if USE_GREEN_COLOR=1
	lda #$F0
	sta $9F23
	stz $9F23
.else
	stz $9F23
	lda #$0F
	sta $9F23
.endif
.else;;;
	; Doing this now because I ran out of time
	lda _are_we_doing_game_cycles
	sta $9F23
	stz $9F23
.endif;;;

	stz _test_col
	dec _test_col

	stz _are_we_doing_game_cycles

	; Reset backdrop so we don't have to in vblank
	stz $9F25					; ADDRSEL=0
	stz $9F20					; backdrop color address $1FA00

	; Now, wait for horizontal blank to set the backdrop color to black. In an ideal world, this would happen 
	; on the previous scanline of the interrupted line and outside the visible screen
	;
	; Let's choose scanline SCNALINE_TEST, which can be weird if the scanline is set to even of odd and scroll registers are written to
	lda #<SCNALINE_TEST
	sta $9F28
	lda #<(%00000011|((SCNALINE_TEST&$100)>>1))	; Top bit of scanline and enable hblank+vblank
	sta $9F26

	jmp @exit_isr
